@@ -31,7 +31,7 @@ from time import gmtime, strftime
 
 from swiftclient import RequestException
 from swiftclient.utils import config_true_value, generate_temp_url, \
-    prt_bytes, JSONableIterable
+    prt_bytes, parse_timeout, JSONableIterable
 from swiftclient.multithreading import OutputManager
 from swiftclient.exceptions import ClientException
 from swiftclient import __version__ as client_version
@@ -227,7 +227,7 @@ def st_delete(parser, args, output_manager, return_parser=False):
                         output_manager.error('Error Deleting: {0}: {1}'
                                              .format(p, r['error']))
         except SwiftError as err:
-            output_manager.error(err.value)
+            output_manager.error_with_txn_id(err)
 
 
 st_download_options = '''[--all] [--marker <marker>] [--prefix <prefix>]
@@ -484,11 +484,13 @@ def st_download(parser, args, output_manager, return_parser=False):
                                     "Object '%s/%s' not found", container, obj)
                                 continue
                         output_manager.error(
-                            "Error downloading object '%s/%s': %s",
-                            container, obj, error)
+                            "Error downloading object '%s/%s': %s\n"
+                            "Failed Transaction ID: %s",
+                            container, obj, error,
+                            getattr(error, 'transaction_id', 'unknown'))
 
         except SwiftError as e:
-            output_manager.error(e.value)
+            output_manager.error_with_txn_id(e)
         except Exception as e:
             output_manager.error(e)
 
@@ -670,7 +672,7 @@ def st_list(parser, args, output_manager, return_parser=False):
                         prt_bytes(totals['bytes'], human))
 
         except SwiftError as e:
-            output_manager.error(e.value)
+            output_manager.error_with_txn_id(e)
 
 
 st_stat_options = '''[--lh] [--header <header:value>]
@@ -754,21 +756,21 @@ def st_stat(parser, args, output_manager, return_parser=False):
                                     items, headers, output_manager
                                 )
                             else:
-                                raise(stat_result["error"])
+                                raise stat_result["error"]
                     else:
                         output_manager.error(
                             'Usage: %s stat %s\n%s', BASENAME,
                             st_stat_options, st_stat_help)
 
         except SwiftError as e:
-            output_manager.error(e.value)
+            output_manager.error_with_txn_id(e)
 
 
 st_post_options = '''[--read-acl <acl>] [--write-acl <acl>] [--sync-to <sync-to>]
                   [--sync-key <sync-key>] [--meta <name:value>]
                   [--header <header>]
                   [<container> [<object>]]
-'''
+'''  # noqa
 
 st_post_help = '''
 Updates meta information for the account, container, or object.
@@ -864,10 +866,10 @@ def st_post(parser, args, output_manager, return_parser=False):
                 else:
                     result = swift.post(container=container)
             if not result["success"]:
-                raise(result["error"])
+                raise result["error"]
 
         except SwiftError as e:
-            output_manager.error(e.value)
+            output_manager.error_with_txn_id(e)
 
 
 st_copy_options = '''[--destination </container/object>] [--fresh-metadata]
@@ -972,14 +974,14 @@ def st_copy(parser, args, output_manager, return_parser=False):
                 return
 
         except SwiftError as e:
-            output_manager.error(e.value)
+            output_manager.error_with_txn_id(e)
 
 
 st_upload_options = '''[--changed] [--skip-identical] [--segment-size <size>]
                     [--segment-container <container>] [--leave-segments]
                     [--object-threads <thread>] [--segment-threads <threads>]
-                    [--meta <name:value>] [--header <header>]
-                    [--use-slo] [--ignore-checksum] [--skip-container-put]
+                    [--meta <name:value>] [--header <header>] [--use-slo]
+                    [--use-dlo] [--ignore-checksum] [--skip-container-put]
                     [--object-name <object-name>]
                     <container> <file_or_directory> [<file_or_directory>] [...]
 '''
@@ -1024,8 +1026,11 @@ Optional arguments:
                         repeated. Example: -H "content-type:text/plain"
                          -H "Content-Length: 4000".
   --use-slo             When used in conjunction with --segment-size it will
-                        create a Static Large Object instead of the default
-                        Dynamic Large Object.
+                        create a Static Large Object. Deprecated; this is now
+                        the default behavior when the cluster supports it.
+  --use-dlo             When used in conjunction with --segment-size it will
+                        create a Dynamic Large Object. May be useful with old
+                        swift clusters.
   --ignore-checksum     Turn off checksum validation for uploads.
   --skip-container-put  Assume all necessary containers already exist; don't
                         automatically try to create them.
@@ -1087,10 +1092,13 @@ def st_upload(parser, args, output_manager, return_parser=False):
         ' This option may be repeated. Example: -H "content-type:text/plain" '
         '-H "Content-Length: 4000"')
     parser.add_argument(
-        '--use-slo', action='store_true', default=False,
+        '--use-slo', action='store_true', default=None,
         help='When used in conjunction with --segment-size, it will '
-        'create a Static Large Object instead of the default '
-        'Dynamic Large Object.')
+        'create a Static Large Object.')
+    parser.add_argument(
+        '--use-dlo', action='store_false', dest="use_slo", default=None,
+        help='When used in conjunction with --segment-size, it will '
+        'create a Dynamic Large Object.')
     parser.add_argument(
         '--object-name', dest='object_name',
         help='Upload file and name object to <object-name> or upload dir and '
@@ -1162,7 +1170,7 @@ def st_upload(parser, args, output_manager, return_parser=False):
         return
 
     if from_stdin:
-        if not options['use_slo']:
+        if options['use_slo'] is None:
             options['use_slo'] = True
         if not options['segment_size']:
             options['segment_size'] = DEFAULT_STDIN_SEGMENT
@@ -1264,7 +1272,7 @@ def st_upload(parser, args, output_manager, return_parser=False):
                                 "to chunk the object")
 
         except SwiftError as e:
-            output_manager.error(e.value)
+            output_manager.error_with_txn_id(e)
 
 
 st_capabilities_options = '''[--json] [<proxy_url>]
@@ -1326,7 +1334,7 @@ def st_capabilities(parser, args, output_manager, return_parser=False):
                 del capabilities['swift']
                 _print_compo_cap('Additional middleware', capabilities)
         except SwiftError as e:
-            output_manager.error(e.value)
+            output_manager.error_with_txn_id(e)
 
 
 st_info = st_capabilities
@@ -1514,7 +1522,7 @@ st_bash_completion_help = '''Retrieve command specific flags used by bash_comple
 
 Optional positional arguments:
   <command>           Swift client command to filter the flags by.
-'''.strip('\n')
+'''.strip('\n')  # noqa
 
 
 st_bash_completion_options = '''[command]
@@ -1630,14 +1638,17 @@ def prompt_for_password():
 def parse_args(parser, args, enforce_requires=True):
     options, args = parser.parse_known_args(args or ['-h'])
     options = vars(options)
-    if enforce_requires and (options.get('debug') or options.get('info')):
+    if enforce_requires and (options.get('debug') or
+                             options.get('debug_with_secrets') or
+                             options.get('info')):
         logging.getLogger("swiftclient")
-        if options.get('debug'):
+        if options.get('debug') or options.get('debug_with_secrets'):
             logging.basicConfig(level=logging.DEBUG)
             logging.getLogger('iso8601').setLevel(logging.WARNING)
-            client_logger_settings['redact_sensitive_headers'] = False
         elif options.get('info'):
             logging.basicConfig(level=logging.INFO)
+    client_logger_settings['redact_sensitive_headers'] = not (
+        enforce_requires and options.get('debug_with_secrets'))
 
     if args and options.get('help'):
         _help = globals().get('st_%s_help' % args[0])
@@ -1724,6 +1735,11 @@ def add_default_args(parser):
                         default=False, help='Show the curl commands and '
                         'results of all http queries regardless of result '
                         'status.')
+    parser.add_argument('--debug-with-secrets', action='store_true',
+                        dest='debug_with_secrets', default=False,
+                        help='Show the curl commands and '
+                        'results of all http queries regardless of result '
+                        'status (without truncating auth tokens).')
     parser.add_argument('--info', action='store_true', dest='info',
                         default=False, help='Show the curl commands and '
                         'results of all http queries which return an error.')
@@ -1746,7 +1762,7 @@ def add_default_args(parser):
     parser.add_argument('-K', '--key', dest='key',
                         default=environ.get('ST_KEY'),
                         help='Key for obtaining an auth token.')
-    parser.add_argument('-T', '--timeout', type=int, dest='timeout',
+    parser.add_argument('-T', '--timeout', type=parse_timeout, dest='timeout',
                         default=None,
                         help='Timeout in seconds to wait for response.')
     parser.add_argument('-R', '--retries', type=int, default=5, dest='retries',
@@ -1950,8 +1966,8 @@ def main(arguments=None):
     parser = argparse.ArgumentParser(
         add_help=False, formatter_class=HelpFormatter, usage='''
 %(prog)s [--version] [--help] [--os-help] [--snet] [--verbose]
-             [--debug] [--info] [--quiet] [--auth <auth_url>]
-             [--auth-version <auth_version> |
+             [--debug] [--debug-with-secrets] [--info] [--quiet]
+             [--auth <auth_url>] [--auth-version <auth_version> |
                  --os-identity-api-version <auth_version> ]
              [--user <username>]
              [--key <api_key>] [--retries <num_retries>]
